/*
 * Web Designer
 * Copyright (C) 1999-2000  LADISLAS Willhem
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author's email : zillprog@freesurf.fr
 * 
 * snailmail : LADISLAS Willhem
 * 2, rue du Serpolet
 * 95490 VAUREAL
 * FRANCE
 */

/*
 * This file was written by Gergely Nagy <algernon@madhouse.lonyay.edu.hu>
 * It contains only two functions: get_full_line. It reads a line from an opened
 * file, and returns it. Meanwhile it allocates the memory too, so the
 * returned line MUST be freed.
 * The other function (const2str) is more simple: you give it a string paramater,
 * and it returns a pointer to another, containing the same thing. It allocates
 * the memory needed, so remember to free() it...
 */
#include <stdio.h>
#include <stdlib.h>
#include "common_headers.h"

#define NEWLINE 10

/*
 * This one reads one line from a file. A line is terminated by a newline or EOF.
 */
char *get_full_line(FILE *fptr)
{
	char *line;
	int cur_chr=0;
	long len=1;

#ifdef DEBUG
	fprintf ( stderr, "(getline.c) [get_full_line] start\n" );
#endif
	while ( ( cur_chr = fgetc ( fptr ) ) != NEWLINE && ! feof ( fptr ) )
	 {
	 	len++;
	 }
	fseek ( fptr, -len, SEEK_CUR );
	if ( ( line = (char *)malloc ( len + 10 ) ) == NULL )
	 {
#ifdef DEBUG
	 	fprintf ( stderr, "(getline.c) [get_full_line] failed to allocate %i bytes of memory for line\n", (int)len+10 );
#endif
	 	return NULL;
	 }
#ifdef DEBUG
	fprintf ( stderr, "(getline.c) [get_full_line] allocated %i bytes of memory for line\n", (int)len+10 );
#endif
	strcpy ( line, "" );
	while ( ( cur_chr = fgetc ( fptr ) ) != NEWLINE && ! feof ( fptr ) )
	 {
	 	sprintf ( line, "%s%c", line, cur_chr );
	 }
	if ( ( cur_chr = fgetc ( fptr ) ) != NEWLINE && ! feof (fptr) )
	  {
	  	fseek(fptr,-1,SEEK_CUR);
	  }
#ifdef DEBUG
	fprintf ( stderr, "(getline.c) [get_full_line] end\n" );
#endif
	return line;
}

/*
 * This reserves memory for a string, copies it paramater to that, and returns it.
 * Useful when setting up dynamically allocated strings.
 * (see preferences.c for an example)
 */
char *const2str ( char *string )
{
	char *str;

#ifdef DEBUG
 	fprintf ( stderr, "(getline.c) [const2str] start\n" );
#endif

	if ( string == NULL )
	{
#ifdef DEBUG
		fprintf ( stderr, "(getline.c) [const2str] end: NULL\n");
#endif
		return NULL;
	}

	if ( ( str = (char *)malloc ( strlen ( string ) + 16 ) ) == NULL )
	 {
#ifdef DEBUG
	 	fprintf ( stderr, "(getline.c) [const2str] failed to allocate %i bytes of memory\n", strlen ( string ) + 16 );
#endif
	 	return NULL;
	 }
#ifdef DEBUG
	fprintf (stderr, "(getline.c) [const2str] allocated %i bytes of memory\n", strlen ( string ) + 16 );
#endif

	strcpy(str, string);

#ifdef DEBUG
 	fprintf ( stderr, "(getline.c) [const2str] end\n" );
#endif
	return str;
}
