/*
 * Web Designer
 * Copyright (C) 1999-2000  LADISLAS Willhem
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author's email : zillprog@freesurf.fr
 * 
 * snailmail :      LADISLAS Willhem
 * 2, rue du Serpolet
 * 95490 VAUREAL
 * FRANCE
 */

/*
 * This file was put together by Gergely Nagy <algernon@madhouse.lonyay.edu.hu>
 * It contains some functions related to recent file handling.
 *
 * FIXME: Underscores should be removed from filenames! Or they should be escaped or something...
 */
#include "common_headers.h"

static void recent_open ( gint fileno );
static gint recent_contains ( char *filename );
static void recent_cleanup ( void );
static void recent_remove ( gint fileno );
static void recent_error ( void );

/*
 * This deletes the recent file entries from the File menu...
 */
void recent_cleanup ( void )
{
	int i=0;
	char *s;

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_cleanup] start\n");
#endif

	gtk_item_factory_delete_item ( gtk_item_factory_from_widget ( menubar ), _("/File/Recent files.../none") );

	for ( i = 0; i < 11; i++ )
	{
		if ( preferences.recent_files[i] != NULL ) {
	  		if ( ( s = (char *) malloc ( strlen ( getname ( preferences.recent_files[i] ) ) + strlen (_("/File/Recent files.../%d - %s")) + 16 ) ) == NULL )
			{
#ifdef DEBUG
				fprintf(stderr,"(recent.c) [recent_cleanup] failed to allocate %i bytes of memory\n", strlen ( getname (preferences.recent_files[i] ) ) + strlen ( _("/File/Recent files.../%d - %s") ) + 16 );
#endif
				return;
			}
#ifdef DEBUG
			fprintf(stderr,"(recent.c) [recent_cleanup] allocated %i bytes of memory for s\n", strlen ( getname ( preferences.recent_files[i] ) ) + strlen (_("/File/Recent files.../%d - %s")) + 16 );
#endif
			sprintf(s, _("/File/Recent files.../%d - %s"), i+1, getname ( preferences.recent_files[i] ));

			gtk_item_factory_delete_item ( gtk_item_factory_from_widget ( menubar ), s );

			free ( s );
#ifdef DEBUG
			fprintf(stderr,"(recent.c) [recent_cleanup] freed s\n");
#endif
		}
	}

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_cleanup] end\n");
#endif
}

/*
 * This one displays the recent files in the /File/Recent files... submenu.
 */
void recent_display ( void )
{
	int i=0;
	GtkItemFactoryEntry this_item;
	char *s;

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_display] start\n");
#endif
	recent_cleanup ();

	this_item.accelerator = NULL;
	this_item.callback = recent_open;
	this_item.callback_action = 0;
	this_item.item_type = NULL;

	while ( preferences.recent_files[i] != NULL && i < preferences.max_recent_files ) 
	{
  		if ( ( s = (char *) malloc ( strlen ( getname ( preferences.recent_files[i] ) ) + strlen (_("/File/Recent files.../%d - %s")) + 16 ) ) == NULL )
		{
#ifdef DEBUG
			fprintf(stderr,"(recent.c) [recent_display] failed to allocate %i bytes of memory\n", strlen ( getname ( preferences.recent_files[i] ) ) + strlen (_("/File/Recent files.../%d - %s")) + 16 );
#endif
			return;
		}
#ifdef DEBUG
		fprintf(stderr,"(recent.c) [recent_display] allocated %i bytes of memory for s\n", strlen ( getname ( preferences.recent_files[i] ) ) + strlen (_("/File/Recent files.../%d - %s")) + 16 );
#endif
		sprintf(s, _("/File/Recent files.../%d - %s"), i+1, getname ( preferences.recent_files[i] ));
	  	this_item.path = s;
		this_item.callback_action = i+1;

		if ( gtk_item_factory_get_item ( gtk_item_factory_from_widget ( menubar ), s ) == NULL )
		  	add_menu_item ( menubar, &this_item );

		free ( s );
#ifdef DEBUG
		fprintf(stderr,"(recent.c) [recent_display] freed s\n");
#endif
		i++;
	}

	if ( preferences.recent_files[0] == NULL || preferences.max_recent_files == 0 )
	{
		this_item.path=_("/File/Recent files.../none");
		this_item.callback_action = 0;
		this_item.callback = recent_error;
		add_menu_item ( menubar, &this_item );
	}

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_display] end\n");
#endif
}

/*
 * This one is used to add files to the
 * recent files array. It shifts the whole thing,
 * and adds the new file as the first element.
 */
void recent_add ( char *filename )
{
	int i;

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_add] start\n");
#endif

	recent_cleanup ();

	if ( ( i = recent_contains ( filename ) ) > -1 )
	{
		recent_remove(i);
	}

	for ( i=10; i>0; i--)
	{
		if ( preferences.recent_files[i-1] != NULL)
		{
			preferences.recent_files[i] = const2str(preferences.recent_files[i-1]);
			free ( preferences.recent_files[i-1] );
		}
		preferences.recent_files[i-1]=NULL;
	}

	preferences.recent_files[0] = const2str ( filename );

	recent_display ();

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_add] end\n");
#endif
}

/*
 * This will be used to open a recent file...
 */
void recent_open ( gint fileno )
{
#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_open] start\n");
#endif
	if ( preferences.recent_files[fileno-1] == NULL )
#ifdef DEBUG
	{
		fprintf (stderr,"(recent.c) [recent_open] end: NULL paramater\n");
#endif
		return;
#ifdef DEBUG
	}
#endif
	openfile2(preferences.recent_files[fileno-1], FALSE, FALSE);
#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_open] end\n");
#endif
}

/*
 * This one prints an error message indicating that the user clicked 'none'...
 */
void recent_error ( void )
{
#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_error] start\n");
#endif
	merror ( _("Cannot open nothing!") );
#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_error] end\n");
#endif
}

/*
 * This returns the position where the filename
 * is found in the recent_files array. Or -1 if
 * not found.
 */
gint recent_contains ( char *filename )
{
	int i;

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_contains] start\n");
#endif
	for ( i = 0; i < preferences.max_recent_files ; i++ )
	{
		if ( preferences.recent_files [i] != NULL )
			if ( strstr ( preferences.recent_files[i], filename ) == preferences.recent_files [i] )
#ifdef DEBUG
			{
				fprintf (stderr, "(recent.c) [recent_contains] end\n");
#endif
				return i;
#ifdef DEBUG
			}
#endif
	}
#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_contains] end\n");
#endif
	return -1;
}

/*
 * This one deletes and entry from the recent_files list.
 */
void recent_remove ( gint fileno )
{
	int i;

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_remove] start\n");
#endif

	for ( i=fileno; i < 11; i ++ )
	{
		if ( preferences.recent_files[i] != NULL)
		{
			free ( preferences.recent_files[i] );
			preferences.recent_files[i]=const2str(preferences.recent_files[i+1]);
		}
	}

#ifdef DEBUG
	fprintf (stderr, "(recent.c) [recent_remove] end\n");
#endif
}
